root = this


class root.Deckchair.Views.Widget extends root.Deckchair.Libraries.Backbone.View

  events:
    'click [data-deckchair-widget-action="prev"]': 'clickPrev'
    'click [data-deckchair-widget-action="next"]': 'clickNext'


  initialize: ->
    _.bindAll @
    raw_widget_options = @$el.data()
    @camera_id = raw_widget_options.deckchairWidget

    @widget_options = $.extend(true, {}, root.Deckchair.Configuration.widget.options)
    @widget_options.resize_mode = raw_widget_options.deckchairWidgetResizeMode if raw_widget_options.deckchairWidgetResizeMode
    @widget_options.width = raw_widget_options.deckchairWidgetWidth if raw_widget_options.deckchairWidgetWidth
    @widget_options.height = raw_widget_options.deckchairWidgetHeight if raw_widget_options.deckchairWidgetHeight

    @define()
    @build()
    @render()


  define: ->
    @direction = 'older'
    @camera = new root.Deckchair.Models.Camera
      id: @camera_id
    @camera.on 'sync', @cameraSynced
    @camera.get('images').on 'sync', @removeOutOfBoundsModels
    @camera.fetch
      update: true

    @$container = $("<div class='deckchair-widget-container' />")
    @$image_container = $("<div class='deckchair-widget-image-container' />")#.css
#      width: @widget_options.width
#      background: '#ccc'

    @$controls = $ """
      <div class="deckchair-widget-controls">
        <div class="deckchair-widget-controls-prev">
          <a href="#" data-deckchair-widget-action="prev">Prev</a>
        </div>

        <div class="deckchair-widget-controls-next">
          <a href="#" data-deckchair-widget-action="next">Next</a>
        </div>
      </div>
      """

    @$hud = $ """
      <div class="deckchair-hud" />
      """

    @$info_date = $ """
      <div class="deckchair-widget-date" />
      """

    @$info_time = $ """
      <div class="deckchair-widget-time" />
      """



  build: ->
    @$el.html ''
    @$el.append @$container
    @$container.append @$image_container
    @$container.append @$controls

    @$hud.append @$info_date
    @$hud.append @$info_time
    @$container.append @$hud


  render: ->
    return @


  createViewForImage: (image_model, options={}) ->
    $.extend true, options,
      model: image_model
      widget_options: @widget_options
      widget_view: @

    image_model.view = new root.Deckchair.Views.WidgetImage( options )
    return image_model.view


  preloadSurrounding: (image, next_or_prev, count=1) ->
    image_index = image.indexInCollection()

    for i in [1..(count+1)]
      image_index_to_preload = image_index - i if next_or_prev == 'next'
      image_index_to_preload = image_index + i if next_or_prev == 'prev'

      image_to_preload = image.collection.at( image_index_to_preload )

      if image_to_preload
        unless image_to_preload.view && image_to_preload.view.imageIsLoaded
          @createViewForImage image_to_preload


  showImage: (new_image_model) ->
    unless new_image_model.view
      @createViewForImage new_image_model

    @image.view.$el.detach() if @image
    @$image_container.append new_image_model.view.el
    @image = new_image_model

    taken = @image.get('taken')

    @$info_date.text( moment(taken).fromNow() )
    @$info_time.text( moment(taken).calendar() )
#    @$info_date.text( "#{taken.getDate()}.#{taken.getMonth()}.#{taken.getFullYear()}" )
#    @$info_time.text( "#{taken.getHours()}:#{taken.getMinutes()}" )


  cameraSynced: (e) ->
    first_image = @camera.get('images').at(0)
    @showImage first_image
    @preloadSurrounding( first_image, 'prev', 2 )


  clickPrev: (e) ->
    @direction == 'older'
    e.preventDefault()
    @setContainerSize()

    unless @image.is_first
      prev_image = @image.getPrev()
      if prev_image
        @showImage prev_image
        @preloadSurrounding( prev_image, 'prev', 5 )
    @bufferIfNearCollectionBoundary()


  clickNext: (e) ->
    @direction == 'newer'
    e.preventDefault()
    @setContainerSize()

    unless @image.is_last
      next_image = @image.getNext()
      if next_image
        @showImage next_image
        @preloadSurrounding( next_image, 'next', 5 )
    @bufferIfNearCollectionBoundary()


  bufferIfNearCollectionBoundary: () ->
    current_index = @image.indexInCollection()
    collection_length = @camera.get('images').length

    # If we are within x places of the end of the collection
    # And the last element is not the oldest available image
    if current_index >= (collection_length - 10) and @camera.get('images').last().get('is_first') == false
      @camera.get('images').fetchPrev @image,
        update: true
        remove: false

    # If we are within x places of the start of the collection
    # And the first element is not the newest available image
    if current_index <= (10) and @camera.get('images').first().get('is_last') == false
      @camera.get('images').fetchNext @image,
        update: true
        remove: false


  removeOutOfBoundsModels: () ->
    unless @image
      return

    current_index = @image.indexInCollection()
    collection = @camera.get('images')

    if collection.length > 100
      if @direction == 'older'
        unused_range_start = 0
        unused_range_end = current_index - 10
      else
        unused_range_start = current_index + 10
        unused_range_end = collection.length - 1

      images_to_remove = collection.slice(unused_range_start, unused_range_end)

      collection.remove images_to_remove
      for image in images_to_remove
        image.view.remove() if image.view


  setContainerSize: () ->
    unless @container_set
      if @image
        @$image_container.css
          width: @image.view.$el.width()
          height: @image.view.$el.height()
        @container_set = true
